@@ -2,8 +2,28 @@
 //!
 //! For help on how to get started, take a look at the [examples](https://github.com/elast0ny/shared_memory-rs/tree/master/examples) !
 
+// Allow dependents to disable logging through the "logging" feature
+cfg_if::cfg_if! {
+    if #[cfg(feature = "logging")] {
+        pub(crate) use log;
+    } else {
+        #[allow(unused_macros)]
+        #[macro_use]
+        pub (crate) mod log {
+            macro_rules! trace (($($tt:tt)*) => {{}});
+            macro_rules! debug (($($tt:tt)*) => {{}});
+            macro_rules! info (($($tt:tt)*) => {{}});
+            macro_rules! warn  (($($tt:tt)*) => {{}});
+            macro_rules! error (($($tt:tt)*) => {{}});
+        }
+    }
+}
+
+#[allow(unused_imports)]
+use crate::log::*;
+
 use std::fs::{File, OpenOptions};
-use std::io::{Read, Write};
+use std::io::{ErrorKind, Read, Write};
 
 use std::fs::remove_file;
 use std::path::{Path, PathBuf};
@@ -39,6 +59,7 @@ impl Drop for ShmemConf {
         // Delete the flink if we are the owner of the mapping
         if self.owner {
             if let Some(flink_path) = self.flink_path.as_ref() {
+                debug!("Deleting file link {}", flink_path.to_string_lossy());
                 let _ = remove_file(flink_path);
             }
         }
@@ -100,17 +121,22 @@ impl ShmemConf {
                     match os_impl::create_mapping(&cur_id, self.size) {
                         Err(ShmemError::MappingIdExists) => continue,
                         Ok(m) => break m,
-                        Err(e) => return Err(e),
+                        Err(e) => {
+                            return Err(e);
+                        }
                     };
                 }
             }
             Some(ref specific_id) => os_impl::create_mapping(specific_id, self.size)?,
         };
+        debug!("Created shared memory mapping '{}'", mapping.unique_id);
 
         // Create flink
         if let Some(ref flink_path) = self.flink_path {
+            debug!("Creating file link that points to mapping");
             let mut open_options: OpenOptions = OpenOptions::new();
             open_options.write(true);
+
             if self.overwrite_flink {
                 open_options.create(true).truncate(true);
             } else {
@@ -119,18 +145,23 @@ impl ShmemConf {
 
             match open_options.open(flink_path) {
                 Ok(mut f) => {
-                    // Write the os_id in the flink
+                    // write the shmem uid asap
                     if let Err(e) = f.write(mapping.unique_id.as_bytes()) {
+                        let _ = std::fs::remove_file(flink_path);
                         return Err(ShmemError::LinkWriteFailed(e));
                     }
                 }
-                Err(e) => {
-                    return Err(match e.kind() {
-                        std::io::ErrorKind::AlreadyExists => ShmemError::LinkExists,
-                        _ => ShmemError::LinkCreateFailed(e),
-                    });
+                Err(e) if e.kind() == ErrorKind::AlreadyExists => {
+                    return Err(ShmemError::LinkExists)
                 }
-            };
+                Err(e) => return Err(ShmemError::LinkCreateFailed(e)),
+            }
+
+            debug!(
+                "Created file link '{}' with id '{}'",
+                flink_path.to_string_lossy(),
+                mapping.unique_id
+            );
         }
 
         self.owner = true;
@@ -146,46 +177,51 @@ impl ShmemConf {
     pub fn open(mut self) -> Result<Shmem, ShmemError> {
         // Must at least have a flink or an os_id
         if self.flink_path.is_none() && self.os_id.is_none() {
+            debug!("Open called with no file link or unique id...");
             return Err(ShmemError::NoLinkOrOsId);
         }
 
-        // Get the os_id from the flink
-        if let Some(ref flink_path) = self.flink_path {
-            let mut f = match File::open(flink_path) {
-                Ok(f) => f,
-                Err(e) => return Err(ShmemError::LinkOpenFailed(e)),
-            };
-            let mut contents: Vec<u8> = Vec::new();
-            if let Err(e) = f.read_to_end(&mut contents) {
-                return Err(ShmemError::LinkReadFailed(e));
-            }
-            let link_os_id = match String::from_utf8(contents) {
-                Ok(s) => s,
-                Err(_) => return Err(ShmemError::LinkDoesNotExist),
-            };
-            if let Some(os_id) = self.os_id.as_ref() {
-                if *os_id != link_os_id {
-                    return Err(ShmemError::FlinkInvalidOsId);
-                }
+        let mut flink_uid = String::new();
+        let mut retry = 0;
+        loop {
+            let unique_id = if let Some(ref unique_id) = self.os_id {
+                unique_id.as_str()
             } else {
-                self.os_id = Some(link_os_id);
+                let flink_path = self.flink_path.as_ref().unwrap();
+                debug!(
+                    "Open shared memory from file link {}",
+                    flink_path.to_string_lossy()
+                );
+                let mut f = match File::open(flink_path) {
+                    Ok(f) => f,
+                    Err(e) => return Err(ShmemError::LinkOpenFailed(e)),
+                };
+                flink_uid.clear();
+                if let Err(e) = f.read_to_string(&mut flink_uid) {
+                    return Err(ShmemError::LinkReadFailed(e));
+                }
+                flink_uid.as_str()
+            };
+
+            match os_impl::open_mapping(unique_id, self.size) {
+                Ok(m) => {
+                    self.size = m.map_size;
+                    self.owner = false;
+
+                    return Ok(Shmem {
+                        config: self,
+                        mapping: m,
+                    });
+                }
+                // If we got this failing os_id from the flink, try again in case the shmem owner didnt write the full
+                // unique_id to the file
+                Err(ShmemError::MapOpenFailed(_)) if self.os_id.is_none() && retry < 5 => {
+                    retry += 1;
+                    std::thread::sleep(std::time::Duration::from_millis(50));
+                }
+                Err(e) => return Err(e),
             }
         }
-
-        let os_id = match self.os_id.as_ref() {
-            Some(i) => i,
-            None => return Err(ShmemError::NoLinkOrOsId),
-        };
-
-        let mapping = os_impl::open_mapping(os_id)?;
-
-        self.size = mapping.map_size;
-        self.owner = false;
-
-        Ok(Shmem {
-            config: self,
-            mapping,
-        })
     }
 }
 
@@ -204,7 +240,6 @@ impl Shmem {
     ///
     /// Warning : You must ensure at least one process owns the mapping in order to ensure proper cleanup code is ran
     pub fn set_owner(&mut self, is_owner: bool) -> bool {
-        #[cfg(any(target_os = "freebsd", target_os = "linux", target_os = "macos"))]
         self.mapping.set_owner(is_owner);
 
         let prev_val = self.config.owner;
